<?php

    require_once('SabreAMF/Server.php');
    require_once('SabreAMF/AMF3/AbstractMessage.php');
    require_once('SabreAMF/AMF3/AcknowledgeMessage.php');
    require_once('SabreAMF/AMF3/RemotingMessage.php');
    require_once('SabreAMF/AMF3/CommandMessage.php');
    require_once('SabreAMF/AMF3/ErrorMessage.php');
    require_once('SabreAMF/AMF3/ClassException.php');	
    require_once('SabreAMF/AMF3/MethodException.php');

    /**
     * AMF Server
     * 
     * This is the AMF0/AMF3 Server class. Use this class to construct a gateway for clients to connect to 
     *
     * The difference between this server class and the regular server, is that this server is aware of the
     * AMF3 Messaging system, and there is no need to manually construct the AcknowledgeMessage classes.
     * Also, the response to the ping message will be done for you.
     * 
     * @package SabreAMF 
     * @version $Id: CallbackServer.php 1905 2006-08-09 20:10:04Z evert $
     * @copyright 2006 Rooftop Solutions
     * @copyright 2006 Renaun Erickson	 
     * @author Evert Pot <evert@collab.nl> 
     * @author Renaun Erickson (http://renaun.com/blog/)
     * @licence http://www.freebsd.org/copyright/license.html  BSD License (4 Clause)
     * @uses SabreAMF_Server
     * @uses SabreAMF_Message
     * @uses SabreAMF_Const
     */
    class SabreAMF_CallbackServer extends SabreAMF_Server {

        /**
         * onInvokeService
         *
         * @var callback
         */
        public $onInvokeService;
		
        private $baseClassPath;		

        private function handleCommandMessage(SabreAMF_AMF3_CommandMessage $request) {

            switch($request->operation) {

                case SabreAMF_AMF3_CommandMessage::CLIENT_PING_OPERATION :
                    $response = new SabreAMF_AMF3_AcknowledgeMessage($request);
                    break;
                default :
                    throw new Exception('Unknown CommandMessage operation: '  . $request->operation);

            }
            return $response;

        }

        protected function invokeService($service,$source,$method,$data) {

            if (is_callable($this->onInvokeService)) {
                return call_user_func_array($this->onInvokeService,array($service,$source,$method,$data));
            } else {
                throw new Exception('onInvokeService is not defined or not callable');
            }

        }

        /**
         * exec
         * 
         * @return void
         */
        public function setBaseClassPath( $classPath = "services/" ) {
			$this->baseClassPath = $classPath;
		}

        /**
         * exec
         * 
         * @return void
         */
        public function exec() {

            foreach($this->getRequests() as $request) {

                // Default AMFVersion
                $AMFVersion = 0;

                $response = null;

                try {

                    // See if we are dealing with the AMF3 messaging system
                    if (is_object($request['data']) && $request['data'] instanceof SabreAMF_AMF3_AbstractMessage) {
                        
                        $AMFVersion = 3;
                       
                        // See if we are dealing with a CommandMessage
                        if ($request['data'] instanceof SabreAMF_AMF3_CommandMessage) {

                            // Handle the command message 
                            $response = $this->handleCommandMessage($request['data']);
                        }

                        // Is this maybe a RemotingMessage ?
                        if ($request['data'] instanceof SabreAMF_AMF3_RemotingMessage) {
			
							$source = $request['data']->source;
							$method = $request['data']->operation;
							
							// locate source file
							$classFile = str_replace( '.', '/', $source ) . '.php';
							

							try {
								// All class paths will be relative to baseClassPath
								$dirname = realpath("./" . $this->baseClassPath);
								if( is_dir( $dirname ) ) {
									chdir( $dirname );
									//throw new Exception( getcwd() . ' - ' . $dirname, 0 );
								} 

								// Check to see if the source class it exists
								if( !file_exists( $classFile ) ) {
									throw new SabreAMF_AMF3_ClassException( $source );
								}
								
								$includeFlag = @include_once( $classFile );

								$lastPeriod = strrpos( $source, "." );
								if( $lastPeriod > 0 )
									$lastPeriod++;
								$className = substr( $source, $lastPeriod, strlen( $source ) - lastPeriod );
								
								//Check if class exists
								if( !class_exists( $className ) )
									throw new SabreAMF_AMF3_ClassException( $source );
								
								$serviceInstance = new $className;
							} catch( Exception $e ) {
								throw new SabreAMF_AMF3_ClassException( $source );
							}							
							if( !isset( $serviceInstance->methodTable[ $method ] ) ) {
								throw new SabreAMF_AMF3_MethodException( $source, $method );
							}

                           	$response = new SabreAMF_AMF3_AcknowledgeMessage($request['data']);
							$response->body = call_user_func_array( array( $serviceInstance, $method ), $request['data']->body );
							
							if( isset( $this->onInvokeService ) {
	                            $response->body = $this->invokeService($request['data']->destination,$request['data']->source,$request['data']->operation,$request['data']->body);
							}

                        }

                    } else {

                        // We are dealing with AMF0
                        $service = substr($request['target'],0,strrpos($request['target'],'.'));
                        $method  = substr(strrchr($request['target'],'.'),1);
                        
                        $response = $this->invokeService($service,"",$method,$request['data']);

                    }

                    $status = SabreAMF_Const::R_RESULT;

                } catch( SabreAMF_AMF3_ClassException $e ) {
                    // We got an exception somewhere, ignore anything that has happened and send back
                    // exception information
                    

                    switch($AMFVersion) {
                        case 0 :
                            $response = array(
                                'description' => $e->getMessage(),
                                'details'     => false,
                                'line'        => $e->getLine(), 
                                'code'        => $e->getCode(),
                            );
                            break;
                        case 3 :
							$response = new SabreAMF_AMF3_ErrorMessage( $request['data'] );
							$response->faultString = $e->getMessage();
							$response->faultCode   = $e->getCode();
							$response->faultDetail  = $e->getDetail();
                            break;

                    }
                    $status = SabreAMF_Const::R_STATUS;
                } catch( Exception $e ) {

                    // We got an exception somewhere, ignore anything that has happened and send back
                    // exception information
                    

                    switch($AMFVersion) {
                        case 0 :
                            $response = array(
                                'description' => $e->getMessage(),
                                'details'     => false,
                                'line'        => $e->getLine(), 
                                'code'        => $e->getCode(),
                            );
                            break;
                        case 3 :
                            $response = new SabreAMF_AMF3_ErrorMessage($request['data']);
                            $response->faultString = $e->getMessage();
                            $response->faultCode   = $e->getCode();
                            break;

                    }
                    $status = SabreAMF_Const::R_STATUS;
                }

                $this->setResponse($request['response'],$status,$response);

            }
            $this->sendResponse();

        }

    }

?>
